from sage.all import *
from py_pari import *
def foo0():	"""
py_pari -- Python interface to PARI C-library.

This module provides a new interface to the PARI C-library.  It is
modeled on the gp interpreter, but with all the advantages associated
with using a real programming language (Python) instead of a hacked
interpreter.

Here are several advantages of Python/PARI over GP:

      - Improved memory management and garbage collection:

            * Automatic garbage collection!  Each Python/PARI object
              has its own piece of memory allocated from the Python
              heap.  When objects go out of scope the Python memory
              manager deletes them.  (Implementation: I grafted
              genuine memory management onto PARI by creatinging a
              mini "PARI stack" in the Python heap for each GEN that
              is wrapped by a Python object.  In some cases it is
              necessary to copy this GEN back to the main PARI stack
              for computations, but this is very fast.)


            * If the "PARI stack overflows", then py_pari
            *automatically* doubles the PARI stack and your
            computation continues uninterrupted.  The main stack is
            unlikely to overflow because it is only used for very
            small atomic computations; as soon as an object is
            computed it is moved of the stack into its own chunk of
            memory in the Python heap.

      - Exception handling.  try, except, raise
      
      - Genuine object oriented language with user defined classes,
        multiple inheritence, polymorphism, etc.
      
      - Object serialization:  This makes loading and saving objects
        to disk easy.
            >>> a = pari(5)
            >>> import pickle
            >>> s = pickle.dumps(a); 
            >>> z = pickle.loads(s);
            >>> print z
            5
            >>> type(z)
            <type '_py_pari.gen'>
            >>> a == z
            True
            >>> a is z
            False

      - Python: easy access to massive amount of software aleady written 
            in Python, e.g., for compression, networking, 2d and 3d
            graphics, etc.

PROBLEMS/WARNINGS

      - PARI commands run at the C-level and cannot be interrupted from
        the Python interpreter with ctrl-C until they return from C level.
        This is in accord with how the Python interpreter works.
        
      - Some of the PARI error and log messages print to stdout.  This
        can be very annoying, since it might interfere with output the
        user is trying to send to a file.  Possible fixes: (1) Modify 
    
"""

def foo1():	"""
* Make it so when asking for, e.g., the n-th prime, if not enough primes have
  been precomputed, py_pari automatically precomputes enough primes and answers
  the question.  Make py_pari robust like MAGMA, and not lame like gp.

"""

def foo2():	"""
    Initialize the PARI system.
    
    INPUT:
        size -- long, the number of bytes for the initial PARI stack
                (see note below)
        maxprime -- unsigned long, upper limit on a precomputed prime
                    number table  (default: 500000)

    NOTES:
    
        * In py_pari, the PARI stack is different than in gp or the
          PARI C library.  In Python, instead of the PARI stack
          holding the results of all computations, it *only* holds the
          results of an individual computation.  Each time a new
          Python/PARI object is computed, it it copied to its own
          space in the Python heap, and the memory it occupied on the
          PARI stack is freed.  Thus it is not necessary to make the
          stack very large.  Also, unlike in PARI, if the stack does
          overflow, in most cases the PARI stack is automatically
          increased and the relevant step of the computation rerun.

          This design obviously involves some performance penalties
          over the way PARI works, but it scales much better and is
          far more robus for large projects.
          
        * If you do not want prime numbers, put maxprime=2, but be
          careful because many PARI functions require this table.  If
          you get the error message "not enough precomputed primes",
          increase this parameter.
          
    """

def foo3():	"""
    Python extension class that models the PARI GEN type.
    """

def foo4():	"""
            g -- PARI GEN
            b -- pointer to memory chunk where PARI gen lives
                 (if nonzero then this memory is freed when the object
                  goes out of scope)
        """

def foo5():	"""
        Return a *copy* of the nth entry.

        The indexing is 0-based, like in Python.  However, *copying*
        the nth entry instead of returning reference is different
        than what one usually does in Python.  However, we do it
        this way for consistency with the PARI/GP interpreter, which
        does make copies.
        """

def foo6():	"""
        EXAMPLES:
            >>> v = pari(xrange(20))
            >>> v[2:5]
            [2, 3, 4]
            >>> v[:]
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
            >>> v[10:]
            [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
            >>> v[:5]
            [0, 1, 2, 3, 4]
            >>> v
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
            >>> v[-1]
            Traceback (most recent call last):
            ...
            IndexError: index (-1) must be between 0 and 19
            >>> v[:-3]
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]
            >>> v[5:]
            [5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
        """

def foo7():	"""
        Set the nth entry to a reference to y.  
        
        WARNINGS:

            -- There is a known BUG: If v is a vector and entry i of v is a vector,
                        v[i][j] = x
               should set entry j of v[i] to x.  Instead it sets it to nonsense.
               I do not understand why this occurs.  The following is a safe way
               to do the same thing:
                        tmp = v[i]; tmp[j] = x    
        
            -- The indexing is 0-based, like everywhere else in Python, but
               *unlike* in GP/PARI.
            
            -- Assignment sets the nth entry to a reference to y,
               assuming y is an object of type gen.  This is the same
               as in Python, but *different* than what happens in the
               gp interpreter, where assignment makes a copy of y.

            -- Because setting creates references it is *possible* to
               make circular references, unlike in GP.  Do *not* do
               this (see the example below).  If you need circular
               references, work at the Python level (where they work
               well), not the PARI object level.

        EXAMPLES:
            >>> v = pari(range(10))
            >>> v
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
            >>> v[0] = 10
            >>> w = pari([5,8,-20])
            >>> v
            [10, 1, 2, 3, 4, 5, 6, 7, 8, 9]
            >>> v[1] = w
            >>> v
            [10, [5, 8, -20], 2, 3, 4, 5, 6, 7, 8, 9]
            >>> w[0] = -30
            >>> v
            [10, [-30, 8, -20], 2, 3, 4, 5, 6, 7, 8, 9]
            >>> t = v[1]; t[1] = 10   # don't do v[1][1] !!! (because of mysterious BUG...)
            >>> v
            [10, [-30, 10, -20], 2, 3, 4, 5, 6, 7, 8, 9]
            >>> w
            [-30, 10, -20]
            
        Finally, we create a circular reference:
            >>> v = pari([0])
            >>> w = pari([v])
            >>> v
            [0]
            >>> w
            [[0]]
            >>> v[0] = w
            >>> # Now there is a circular reference.  Trying to access v[0] will crash SAGE.

        """

def foo8():	"""
        EXAMPLES:
            >>> f = pari("x^2 + y^3 + x*y")
            >>> f
            x^2 + y*x + y^3
            >>> f.polcoeff(1)
            y
            >>> f.polcoeff(3)
            0
            >>> f.polcoeff(3, "y")
            1
            >>> f.polcoeff(1, "y")
            x
        """

def foo9():	"""
        Comparisons.
        """

def foo10():	"""
        EXAMPLES:
            >>> a = pari(5)
            >>> b = 10
            >>> a < b
            True
            >>> a <= b
            True
            >>> a <= 5
            True
            >>> a > b
            False
            >>> a >= b
            False
            >>> a >= pari(10)
            False
            >>> a == 5
            True
            >>> a is 5
            False
        """

def foo11():	"""
        Return str that might correctly evaluate to a Python-list.
        """

def foo12():	"""
        Return Python int.
        """

def foo13():	"""
        Return a Python list of the PARI gens.  This object must be of
        type t_VEC

        INPUT: None
        OUTPUT:
           list -- Python list whose elements are the
                   elements of the input gen.
        EXAMPLES:
            >>> v=pari([1,2,3,10,102,10])
            >>> w = v.python_list()
            >>> w
            [1, 2, 3, 10, 102, 10]
            >>> type(w[0])
            <type '_py_pari.gen'>
        """

def foo14():	"""
        Return Python eval of self.
        """

def foo15():	"""
        Return Python long.
        """

def foo16():	"""
        Return Python float.
        """

def foo17():	"""
        isprime(x, flag=0): Returns True if x is a PROVEN prime
        number, and False otherwise. 

        INPUT:
            flag -- int 
                    0 (default): use a combination of algorithms.
                    1: certify primality using the Pocklington-Lehmer Test.
                    2: certify primality using the APRCL test.
        OUTPUT:
            bool -- True or False
        EXAMPLES:
            >>> pari(9).isprime()
            False
            >>> pari(17).isprime()
            True
            >>> n = pari(561)    # smallest Carmichael number
            >>> n.isprime()      # not just a pseudo-primality test!
            False
            >>> n.isprime(1)
            False
            >>> n.isprime(2)
            False
        """

def foo18():	"""

        divrem(x, y, {v}): Euclidean division of x by y giving as a
            2-dimensional column vector the quotient and the
            remainder, with respect to v (to main variable if v is
            omitted).

        >>> 
        """

def foo19():	"""

        lex(x,y): Compare x and y lexicographically (1 if x>y, 0 if
            x==y, -1 if x<y)
        
        """

def foo20():	"""
        max(x,y): Return the maximum of x and y.
        """

def foo21():	"""
        min(x,y): Return the minumum of x and y.
        """

def foo22():	"""
        shift(x,n): shift x left n bits if n>=0, right -n bits if n<0.
        """

def foo23():	"""
        shiftmul(x,n): Return the product of x by 2^n (n>=0 or n<0).
        """

def foo24():	"""
        sign(x): Return the sign of x, where x is of type integer, real
            or fraction.
        """

def foo25():	"""
        vecmax(x): Return the maximum of the elements of the vector/matrix x,
        """

def foo26():	"""
        vecmin(x): Return the maximum of the elements of the vector/matrix x,
        """

def foo27():	"""
        Col(x): Transforms the object x into a column vector.
        
        The vector will have only one component, except in the
        following cases:

           * When x is a vector or a quadratic form, the resulting
             vector is the initial object considered as a column
             vector.

           * When x is a matrix, the resulting vector is the column of
             row vectors comprising the matrix.

           * When x is a character string, the result is a column of
             individual characters.

           * When x is a polynomial, the coefficients of the vector
             start with the leading coefficient of the polynomial.

           * When x is a power series, only the significant
             coefficients are taken into account, but this time by
             increasing order of degree.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari(1.5).Col()
            [1.5000000000000000000000000000]~
            >>> pari([1,2,3,4]).Col()
            [1, 2, 3, 4]~
            >>> pari('[1,2; 3,4]').Col()
            [[1, 2], [3, 4]]~
            >>> pari('"SAGE"').Col()
            ["S", "A", "G", "E"]~
            >>> pari('3*x^3 + x').Col()
            [3, 0, 1, 0]~
            >>> pari('x + 3*x^3 + O(x^5)').Col()
            [1, 0, 3, 0]~
        """

def foo28():	"""
        List(x): transforms the PARI vector or list x into a list.

        EXAMPLES:
            >>> v = pari([1,2,3])
            >>> v
            [1, 2, 3]
            >>> v.type()
            't_VEC'
            >>> w = v.List()
            >>> w
            List([1, 2, 3])
            >>> w.type()
            't_LIST'
        """

def foo29():	"""
        Mat(x): Returns the matrix defined by x.
        
           * If x is already a matrix, a copy of x is created and
             returned.
           
           * If x is not a vector or a matrix, this function returns a
             1x1 matrix.
        
           * If x is a row (resp. column) vector, this functions
             returns a 1-row (resp. 1-column) matrix, *unless* all
             elements are column (resp. row) vectors of the same
             length, in which case the vectors are concatenated
             sideways and the associated big matrix is returned.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- a PARI matrix

        EXAMPLES:
            >>> x = pari(5)
            >>> x.type()
            't_INT'
            >>> y = x.Mat()
            >>> y
            Mat(5)
            >>> y.type()
            't_MAT'
            >>> x = pari('[1,2;3,4]')
            >>> x.type()
            't_MAT'
            >>> x = pari('[1,2,3,4]')
            >>> x.type()
            't_VEC'
            >>> y = x.Mat()
            >>> y
            Mat([1, 2, 3, 4])
            >>> y.type()
            't_MAT'

            >>> v = pari('[1,2;3,4]').Vec(); v
            [[1, 3]~, [2, 4]~]
            >>> v.Mat()
            [1, 2; 3, 4]
            >>> v = pari('[1,2;3,4]').Col(); v
            [[1, 2], [3, 4]]~
            >>> v.Mat()
            [1, 2; 3, 4]
        """

def foo30():	"""
        Mod(x, y): Returns the object x modulo y, denoted Mod(x, y).

        The input y must be a an integer or a polynomial:

           * If y is an INTEGER, x must also be an integer, a rational
             number, or a p-adic number compatible with the modulus y.

           * If y is a POLYNOMIAL, x must be a scalar (which is not a polmod),
             a polynomial, a rational function, or a power series.

        WARNING: This function is not the same as x % y, the result of
        which is an integer or a polynomial.

        INPUT:
            x -- gen
            y -- integer or polynomial

        OUTPUT:
            gen -- intmod or polmod
            
        EXAMPLES:
            >>> z = pari(3)
            >>> x = z.Mod(pari(7))
            >>> x
            Mod(3, 7)
            >>> x**2
            Mod(2, 7)
            >>> x**100
            Mod(4, 7)
            >>> x.type()
            't_INTMOD'

            >>> f = pari("x^2 + x + 1")
            >>> g = pari("x")
            >>> a = g.Mod(f)
            >>> a
            Mod(x, x^2 + x + 1)
            >>> a*a
            Mod(-x - 1, x^2 + x + 1)
            >>> a.type()
            't_POLMOD'
        """

def foo31():	"""
        Pol(x): convert x (usually a vector or a power series) into a
        polynomial, starting with the leading coefficient

        INPUT:
            x -- gen
            var -- (optional) which variable
        OUTPUT:
            gen -- a polynomial
        EXAMPLES:
            >>> v = pari("[1,2,3,4]")
            >>> f = v.Pol()
            >>> f
            x^3 + 2*x^2 + 3*x + 4
            >>> f*f
            x^6 + 4*x^5 + 10*x^4 + 20*x^3 + 25*x^2 + 24*x + 16
            
            >>> v = pari("[1,2;3,4]")
            >>> v.Pol()
            [1, 3]~*x + [2, 4]~
        """

def foo32():	"""
        Polrev(x): convert x (usually a vector or a power series) into a
        polynomial, starting with the constant

        INPUT:
            x -- gen
        OUTPUT:
            gen -- a polynomial
        EXAMPLES:
            >>> v = pari("[1,2,3,4]")
            >>> f = v.Polrev()
            >>> f
            4*x^3 + 3*x^2 + 2*x + 1
            >>> f*f
            16*x^6 + 24*x^5 + 25*x^4 + 20*x^3 + 10*x^2 + 4*x + 1
            
        Note that Polrev does *not* reverse the coefficients of a polynomial!
            >>> f.Polrev()
            4*x^3 + 3*x^2 + 2*x + 1
            >>> v = pari("[1,2;3,4]")
            >>> v.Polrev()
            [2, 4]~*x + [1, 3]~
        """

def foo33():	"""
        Qfb(a,b,c,{D=0.}): binary quadratic form a*x^2+b*x*y+c*y^2.
            D is optional (0.0 by default) and initializes
            Shanks's distance if b^2-4*a*c>0

        INPUT:
            a -- gen
            b -- gen
            c -- gen
            D -- gen
        OUTPUT:
            gen -- binary quadratic form
        EXAMPLES:
            >>> pari(3).Qfb(7, 2)
            Qfb(3, 7, 2, 0.E-28)
            
        """

def foo34():	"""
        Ser(x,{v=x}): convert x (usually a vector) into a power series
        with variable v, starting with the constant coefficient.

        INPUT:
            x -- gen
            v -- PARI variable (default: x)
        OUTPUT:
            gen -- PARI object of PARI type t_SER
        EXAMPLES:
            >>> x = pari([1,2,3,4,5])
            >>> x.Ser()
            1 + 2*x + 3*x^2 + 4*x^3 + 5*x^4 + O(x^5)
            >>> f = x.Ser('v'); print f
            1 + 2*v + 3*v^2 + 4*v^3 + 5*v^4 + O(v^5)
            >>> pari(1)/f
            1 - 2*v + v^2 + O(v^5)
            >>> pari(1).Ser()
            1 + O(x^16)
        """

def foo35():	"""
        Set(x): convert x into a set, i.e. a row vector of strings in
        increasing lexicographic order.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- a vector of strings in increasing lexicographic order.
        EXAMPLES:
            >>> pari([1,5,2]).Set()
            ["1", "2", "5"]
            >>> pari([]).Set()     # the empty set
            []
            >>> pari([1,1,-1,-1,3,3]).Set()
            ["-1", "1", "3"]
            >>> pari(1).Set()
            ["1"]
            >>> pari('1/(x*y)').Set()
            ["1/(y*x)"]
            >>> pari('["bc","ab","bc"]').Set()
            ["ab", "bc"]
        """

def foo36():	"""
        Str(x): Concatenate the entries of the vector x into a single
        string.  If x is not a t_VEC its print representation is
        returned.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- a PARI gen of type t_STR, i.e., a PARI string
        EXAMPLES:
            >>> pari([1,2,['abc',1]]).Str()
            12[abc, 1]
            >>> pari([1,1,1.54]).Str()
            111.5400000000000000000000000000
            >>> pari([1,1.54,1]).Str()
            11.54000000000000000000000000001
            >>> pari(1).Str()       # 1 is automatically converted to string rep
            1
            >>> x = pari('x')       # PARI variable "x"
            >>> x.Str()             # is converted to string rep.
            x
            >>> x.Str().type()
            't_STR'
        """

def foo37():	"""
        Strchr(x): converts x to a string, translating each integer
        into a character (in ASCII).

        NOTE: Vecsmall is (essentially) the inverse to Strchr().

        INPUT:
            x -- PARI vector of integers
        OUTPUT:
            gen -- a PARI string
        EXAMPLES:
            >>> pari([65,66,123]).Strchr()
            AB{
            >>> pari('"SAGE"').Vecsmall()   # pari('"SAGE"') --> PARI t_STR
            Vecsmall([83, 65, 71, 69])
            >>> _.Strchr()
            SAGE
            >>> pari([83, 65, 71, 69]).Strchr()
            SAGE
        """

def foo38():	"""
        Strexpand(x): Concatenate the entries of the vector x into a
        single string, performing tilde expansion.

        NOTE: I have no clue what the point of this function is. -- William
        """

def foo39():	"""
        Strtex(x): Translates the vector x of PARI gens to TeX format
        and returns the resulting concatenated strings as a PARI t_STR.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- PARI t_STR (string)
        EXAMPLES:
            >>> v=pari('x^2')
            >>> v.Strtex()
            {x^{2}}
            >>> v=pari(['1/x^2','x'])
            >>> v.Strtex()
            {{1}\over{x^{2}}}{x}
            >>> v=pari(['1 + 1/x + 1/(y+1)','x-1'])
            >>> v.Strtex()             # the "\\r" is really "\r" (changed for doctests)
            {{ \left({y + {2}}\over{y + 1}\\right) x + 1}\over{x}}{x - 1}
        """

def foo40():	"""
        Vec(x): Transforms the object x into a vector.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- of PARI type t_VEC
        EXAMPLES:
            >>> pari(1).Vec()
            [1]
            >>> pari('x^3').Vec()
            [1, 0, 0, 0]
            >>> pari('x^3 + 3*x - 2').Vec()
            [1, 0, 3, -2]
            >>> pari([1,2,3]).Vec()
            [1, 2, 3]
            >>> pari('ab').Vec()
            [1, 0]
        """

def foo41():	"""
        Vecsmall(x): transforms the object x into a t_VECSMALL.

        INPUT:
            x -- gen
        OUTPUT:
            gen -- PARI t_VECSMALL
        EXAMPLES:
            >>> pari([1,2,3]).Vecsmall()
            Vecsmall([1, 2, 3])
            >>> pari('"SAGE"').Vecsmall()
            Vecsmall([83, 65, 71, 69])
            >>> pari(1234).Vecsmall()
            Vecsmall([1234])
        """

def foo42():	"""
        binary(x): gives the vector formed by the binary digits of
        abs(x), where x is of type t_INT.

        INPUT:
            x -- gen of type t_INT
        OUTPUT:
            gen -- of type t_VEC
        EXAMPLES:
            >>> pari(0).binary()
            [0]
            >>> pari(-5).binary()
            [1, 0, 1]
            >>> pari(5).binary()
            [1, 0, 1]
            >>> pari(2005).binary()
            [1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1]

            >> pari('"2"').binary()
            Traceback (most recent call last):
            ...
            TypeError: x (=2) must be of type t_INT, but is of type t_STR.
        """

def foo43():	"""
        bitand(x,y): Bitwise and of two integers x and y. Negative
        numbers behave as if modulo some large power of 2.

        INPUT:
            x -- gen  (of type t_INT)
            y -- coercible to gen  (of type t_INT)
        OUTPUT:
            gen -- of type type t_INT
        EXAMPLES:
            >>> pari(8).bitand(4)
            0
            >>> pari(8).bitand(8)
            8
            >>> pari(6).binary()
            [1, 1, 0]
            >>> pari(7).binary()
            [1, 1, 1]
            >>> pari(6).bitand(7)
            6
            >>> pari(19).bitand(-1)
            19
            >>> pari(-1).bitand(-1)
            -1
        """

def foo44():	"""
        bitneg(x,{n=-1}): Bitwise negation of the integer x truncated
        to n bits.  n=-1 (the default) represents an infinite sequence
        of the bit 1.  Negative numbers behave as if modulo some large
        power of 2.

        With n=-1, this function returns -n-1.  With n >= 0, it returns
        a number a such that  a = -n-1 (mod 2^n).

        INPUT:
            x -- gen (t_INT)
            n -- long, default = -1
        OUTPUT:
            gen -- t_INT
        EXAMPLES:
            >>> pari(10).bitneg()
            -11
            >>> pari(1).bitneg()
            -2
            >>> pari(-2).bitneg()
            1
            >>> pari(-1).bitneg()
            0
            >>> pari(569).bitneg()
            -570
            >>> pari(569).bitneg(10)
            454
            >>> 454 % 2**10
            454
            >>> -570 % 2**10
            454
        """

def foo45():	"""
        bitnegimply(x,y): Bitwise negated imply of two integers x and
        y, in other words, x BITAND BITNEG(y). Negative numbers behave
        as if modulo big power of 2.

        INPUT:
            x -- gen  (of type t_INT)
            y -- coercible to gen  (of type t_INT)
        OUTPUT:
            gen -- of type type t_INT
        EXAMPLES:
            >>> pari(14).bitnegimply(0)    
            14
            >>> pari(8).bitnegimply(8)
            0
            >>> pari(8+4).bitnegimply(8)
            4
        """

def foo46():	"""
        bitor(x,y): Bitwise or of two integers x and y. Negative
        numbers behave as if modulo big power of 2.

        INPUT:
            x -- gen  (of type t_INT)
            y -- coercible to gen  (of type t_INT)
        OUTPUT:
            gen -- of type type t_INT
        EXAMPLES:
            >>> pari(14).bitor(0)
            14
            >>> pari(8).bitor(4)
            12
            >>> pari(12).bitor(1)
            13
            >>> pari(13).bitor(1)
            13
        """

def foo47():	"""
        bittest(x, long n): Returns bit number n (coefficient of 2^n in binary)
        of the integer x. Negative numbers behave as if modulo a big power of 2.

        INPUT:
           x -- gen (pari integer)
        OUTPUT:
           bool -- a Python bool
        EXAMPLES:
            >>> x = pari(6)
            >>> x.bittest(0)
            False
            >>> x.bittest(1)
            True
            >>> x.bittest(2)
            True
            >>> x.bittest(3)
            False
            >>> pari(-3).bittest(0)
            True
            >>> pari(-3).bittest(1)
            False
            >>> [pari(-3).bittest(n) for n in range(10)]
            [True, False, True, True, True, True, True, True, True, True]
        """

def foo48():	"""
        bitxor(x,y): Bitwise exclusive or of two integers x and y.
        Negative numbers behave as if modulo big power of 2.
        
        INPUT:
            x -- gen  (of type t_INT)
            y -- coercible to gen  (of type t_INT)
        OUTPUT:
            gen -- of type type t_INT
        EXAMPLES:
            >>> pari(6).bitxor(4)
            2
            >>> pari(0).bitxor(4)
            4
            >>> pari(6).bitxor(0)
            6
        """

def foo49():	"""
        Return the smallest integer >= x.

        INPUT:
           x -- gen
        OUTPUT:
           gen -- integer
        EXAMPLES:
            >>> pari(1.4).ceil()
            2
            >>> pari(-1.4).ceil()
            -1
            >>> pari('x').ceil()
            x
            >>> pari('x^2+5*x+2.2').ceil()
            x^2 + 5*x + 2.2000000000000000000000000000
            >>> pari('3/4').ceil()
            1
        """

def foo50():	"""
        centerlift(x,{v}): Centered lift of x.  This function returns
        exactly the same thing as lift, except if x is an integer mod.

        INPUT:
            x -- gen
            v -- var (default: x)
        OUTPUT:
            gen
        EXAMPLES:
            >>> x = pari(-2).Mod(5)
            >>> x.centerlift()
            -2
            >>> x.lift()
            3
            >>> f = pari('x-1').Mod('x^2 + 1')
            >>> f.centerlift()
            x - 1
            >>> f.lift()
            x - 1
            >>> f = pari('x-y').Mod('x^2+1')
            >>> f
            Mod(x - y, x^2 + 1)
            >>> f.centerlift('x')
            x - y
            >>> f.centerlift('y')
            Mod(x - y, x^2 + 1)
        """

def foo51():	"""
        changevar(gen x, y): change variables of x according to the vector y.

        INPUT:
            x -- gen
            y -- gen (or coercible to gen)
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('x^3+1').changevar(['y'])
            y^3 + 1
            >>> pari('x^3+y^2').changevar(['y','z'])
            y^3 + z^2
            >>> pari('x^3+y^2').changevar(['y','z+1'])
            y^3 + (z^2 + 2*z + 1)
        """

def foo52():	"""
        component(x, long n): Return n'th component of the internal
        representation of x.  This this function is 1-based
        instead of 0-based.

        NOTE: For vectors or matrices, it is simpler to use x[n-1]. For
        list objects such as is output by nfinit, it is easier to use
        member functions.

        INPUT:
            x -- gen
            n -- C long (coercible to)
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari([0,1,2,3,4]).component(1)
            0
            >>> pari([0,1,2,3,4]).component(2)
            1
            >>> pari([0,1,2,3,4]).component(4)
            3
            >>> pari('x^3 + 2').component(1)
            2
            >>> pari('x^3 + 2').component(2)
            0
            >>> pari('x^3 + 2').component(4)
            1
            >>> pari('x').component(0)
            Traceback (most recent call last):
            ...
            IndexError: component (=0) must be between 1 and 2, inclusive
        """

def foo53():	"""
        conj(x): Return the algebraic conjugate of x.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('x+1').conj()
            x + 1
            >>> pari('x+I').conj()
            x - I
            >>> pari('1/(2*x+3*I)').conj()
            1/(2*x - 3*I)
            >>> pari([1,2,'2-I','Mod(x,x^2+1)', 'Mod(x,x^2-2)']).conj()
            [1, 2, 2 + I, Mod(-x, x^2 + 1), Mod(-x, x^2 - 2)]
            >>> pari('Mod(x,x^2-2)').conj()
            Mod(-x, x^2 - 2)
            
            >> pari('Mod(x,x^3-3)').conj()
            Traceback (most recent call last):
            ...
            TypeError: x (=Mod(x, x^3 - 3)) has no conjugate.
        """

def foo54():	"""
        conjvec(x): Returns the vector of all conjugates of the
        algebraic number x.  An algebraic number is a polynomial over
        Q modulo an irreducible polynomial.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('Mod(1+x,x^2-2)').conjvec()
            [-0.41421356237309504880168872420, 2.4142135623730950488016887242]~
            >>> pari('Mod(x,x^3-3)').conjvec()
            [1.4422495703074083823216383108, -0.72112478515370419116081915539 + 1.2490247664834064794131795438*I, -0.72112478515370419116081915539 - 1.2490247664834064794131795438*I]~
        """

def foo55():	"""
        denominator(x): Return the denominator of x.  When x is a
        vector, this is the least common multiple of the denominators
        of the components of x.

        what about poly?
        INPUT:
            x -- gen
        OUTPUT:
            gen 
        EXAMPLES:
            >>> pari('5/9').denominator()
            9
            >>> pari('(x+1)/(x-2)').denominator()
            x - 2
            >>> pari('2/3 + 5/8*x + 7/3*x^2 + 1/5*y').denominator()
            1
            >>> pari('2/3*x').denominator()
            1
            >>> pari('[2/3, 5/8, 7/3, 1/5]').denominator()
            120
        """

def foo56():	"""
        floor(x): Return the floor of x, which is the largest integer <= x.
        This function also works component-wise on polynomials, vectors, etc.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('5/9').floor()
            0
            >>> pari('11/9').floor()
            1
            >>> pari('1.17').floor()
            1
            >>> pari('x').floor()
            x
            >>> pari('x+1.5').floor()
            x + 1.5000000000000000000000000000
            >>> pari('[1.5,2.3,4.99]').floor()
            [1, 2, 4]
            >>> pari('[[1.1,2.2],[3.3,4.4]]').floor()
            [[1, 2], [3, 4]]

            >> pari('"hello world"').floor()
            Traceback (most recent call last):
            ...
            TypeError: x (=hello world) has no floor.
        """

def foo57():	"""
        frac(x): Return the fractional part of x, which is x - floor(x).

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('1.7').frac()
            0.70000000000000000000000000001
            >>> pari('sqrt(2)').frac()
            0.41421356237309504880168872420

            >> pari('sqrt(-2)').frac()
            Traceback (most recent call last):
            ...
            TypeError: Fractional part of 1.4142135623730950488016887242*I not defined.
        """

def foo58():	"""
        imag(x): Return the imaginary part of x.  This function also
        works component-wise.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:
            >>> pari('1+2*I').imag()
            2
            >>> pari('sqrt(-2)').imag()
            1.4142135623730950488016887242
            >>> pari('x+I').imag()
            1
            >>> pari('x+2*I').imag()
            2
            >>> pari('(1+I)*x^2+2*I').imag()
            x^2 + 2
            >>> pari('[1,2,3] + [4*I,5,6]').imag()
            [4, 0, 0]
        """

def foo59():	"""
        length(x): Return the number of non-code words in x.  If x
        is a string, this is the number of characters of x.

        ?? terminator ?? carriage return ??
        """

def foo60():	"""
        lift(x,{v}): Returns the lift of an element of Z/nZ to Z or
        R[x]/(P) to R[x] for a type R if v is omitted.  If v is given,
        lift only polymods with main variable v.  If v does not occur
        in x, lift only intmods.

        INPUT:
            x -- gen
            v -- (optional) variable
        OUTPUT:
            gen 
        EXAMPLES:
            >>> x = pari("x")
            >>> a = x.Mod(x**3 + 17*x + 3)
            >>> a
            Mod(x, x^3 + 17*x + 3)
            >>> b = a**4; b
            Mod(-17*x^2 - 3*x, x^3 + 17*x + 3)
            >>> b.lift()
            -17*x^2 - 3*x

        ??? more examples
        """

def foo61():	"""
        numerator(x): Returns the numerator of x.

        INPUT:
            x -- gen
        OUTPUT:
            gen
        EXAMPLES:

        """

def foo62():	"""
        numtoperm(k, n): Return the permutation number k (mod n!) of n
        letters, where n is an integer.
        
        INPUT:
            k -- gen, integer
            n -- int
        OUTPUT:
            gen -- vector (permutation of {1,...,n})
        EXAMPLES:
        """

def foo63():	"""
        padicprec(x,p): Return the absolute p-adic precision of the object x.

        INPUT:
            x -- gen
        OUTPUT:
            int
        EXAMPLES:
        """

def foo64():	"""
        permtonum(x): Return the ordinal (between 1 and n!) of permutation vector x.
        ??? Huh ???  say more.  what is a perm vector.  0 to n-1 or 1-n.

        INPUT:
            x -- gen (vector of integers)
        OUTPUT:
            gen -- integer
        EXAMPLES:
        """

def foo65():	"""
        precision(x,{n}): Change the precision of x to be n, where n
        is a C-integer). If n is omitted, output the real precision of x.

        INPUT:
            x -- gen
            n -- (optional) int
        OUTPUT:
            nothing
          or
            gen if n is omitted
        EXAMPLES:
        """

def foo66():	"""
        random({N=2^31}): Return a pseudo-random integer between 0 and N-1.

        INPUT:
            N -- gen, integer
        OUTPUT:
            gen -- integer
        EXAMPLES:
        """

def foo67():	"""
        real(x): Return the real part of x.

        INPUT:
            x -- gen
        OUTPUT:
            gen 
        EXAMPLES:
        """

def foo68():	"""
        round(x,estimat=False):  If x is a real number, returns x rounded
        to the nearest integer (rounding up).  If the optional argument
        estimate is True, also returns the binary exponent e of the difference
        between the original and the rounded value (the "fractional part")
        (this is the integer ceiling of log_2(error)).

        When x is a general PARI object, this function returns the result
        of rounding every coefficient at every level of PARI object.
        Note that this is different than what the truncate function
        does (see the example below).

        One use of round is to get exact results after a long
        approximate computation, when theory tells you that the
        coefficients must be integers.
        
        INPUT:
            x -- gen
            estimate -- (optional) bool, False by default
        OUTPUT:
            * if estimate == False, return a single gen.
            * if estimate == True, return rounded verison of x and
              error estimate in bits, both as gens.
        EXAMPLES:
            >>> pari('1.5').round()
            2
            >>> pari('1.5').round(True)
            (2, -1)
            >>> pari('1.5 + 2.1*I').round()
            2 + 2*I
            >>> pari('1.0001').round(True)
            (1, -14)
            >>> pari('(2.4*x^2 - 1.7)/x').round()
            (2*x^2 - 2)/x
            >>> pari('(2.4*x^2 - 1.7)/x').truncate()
            2.4000000000000000000000000000*x
        """

def foo69():	"""
        simplify(x): Simplify the object x as much as possible, and return
        the result.

        A complex or quadratic number whose imaginary part is an exact 0
        (i.e., not an approximate one such as O(3) or 0.E-28) is converted
        to its real part, and a a polynomial of degree 0 is converted to
        its constant term.  Simplification occurs recursively.

        This function is useful before using arithmetic functions, which
        expect integer arguments:
            >>> y = pari('y')
            >>> x = pari('9') + y - y
            >>> x
            9
            >>> x.type()
            't_POL'
            >>> x.factor()
            matrix(0,2)
            >>> pari('9').factor()
            Mat([3, 2])
            >>> x.simplify()
            9
            >>> x.simplify().factor()
            Mat([3, 2])
        
        INPUT:
            x -- gen
        OUTPUT:
            gen 
        EXAMPLES:
            >>> x = pari('1.5 + 0*I')
            >>> x.type()
            't_COMPLEX'
            >>> x.simplify()
            1.5000000000000000000000000000
            >>> y = x.simplify()
            >>> y.type()
            't_REAL'
        """

def foo70():	"""
        sizebyte(x): Return the total number of bytes occupied by the
        complete tree of the object x.
        
        INPUT:
            x -- gen
        OUTPUT:
            int (a Python int)
        EXAMPLES:
            >>> pari('1').sizebyte()
            12
            >>> pari('10').sizebyte()
            12
            >>> pari('10000000000000').sizebyte()
            16
            >>> pari('10^100').sizebyte()
            52
            >>> pari('x').sizebyte()
            36
            >>> pari('x^20').sizebyte()
            264
            >>> pari('[x, 10^100]').sizebyte()
            100
        """

def foo71():	"""

        sizedigit(x): Return a quick estimate for the maximal number of
        decimal digits before the decimal point of any component of x.

        INPUT:
            x -- gen
        OUTPUT:
            int -- Python integer
        EXAMPLES:
            >>> x = pari('10^100')
            >>> x.Str().length()
            101
            >>> x.sizedigit()
            101

        Note that digits after the decimal point are ignored.
            >>> x = pari('1.234')
            >>> x
            1.2340000000000000000000000000
            >>> x.sizedigit()
            1

        The estimate can be one too big:
            >>> pari('7234.1').sizedigit()
            4
            >>> pari('9234.1').sizedigit()
            5
        """

def foo72():	"""
        truncate(x,estimate=False):  Return the truncation of x.
        If estimate is True, also return the number of error bits.
        
        When x is in the real numbers, this means that the part
        after the decimal point is chopped away, e is the binary
        exponent of the difference between the original and truncated
        value (the "fractional part").    If x is a rational
        function, the result is the integer part (Euclidean
        quotient of numerator by denominator) and if requested
        the error estimate is 0.

        When truncate is applied to a power series (in X), it
        transforms it into a polynomial or a rational function with
        denominator a power of X, by chopping away the O(X^k).
        Similarly, when applied to a p-adic number, it transforms it
        into an integer or a rational number by chopping away the
        O(p^k).

        INPUT:
            x -- gen
            estimate -- (optional) bool, which is False by default
        OUTPUT:
            * if estimate == False, return a single gen.
            * if estimate == True, return rounded verison of x and
              error estimate in bits, both as gens.
        OUTPUT:
        EXAMPLES:
            >>> pari('(x^2+1)/x').round()
            (x^2 + 1)/x
            >>> pari('(x^2+1)/x').truncate()
            x
            >>> pari('1.043').truncate()
            1
            >>> pari('1.043').truncate(True)
            (1, -5)
            >>> pari('1.6').truncate()
            1
            >>> pari('1.6').round()
            2
            >>> pari('1/3 + 2 + 3^2 + O(3^3)').truncate()
            34/3
            >>> pari('sin(x+O(x^10))').truncate()
            1/362880*x^9 - 1/5040*x^7 + 1/120*x^5 - 1/6*x^3 + x
            >>> pari('sin(x+O(x^10))').round()   # each coefficient has abs < 1
            x + O(x^10)
        """

def foo73():	"""
        valuation(x,p): Return the valuation of x with respect to p.

        The valuation is the highest exponent of p dividing x.

           * If p is an integer, x must be an integer, an intmod whose
             modulus is divisible by p, a rational number, a p-adic
             number, or a polynomial or power series in which case the
             valuation is the minimal of the valuations of the
             coefficients.

           * If p is a polynomial, x must be a polynomial or a
             rational fucntion.  If p is a monomial then x may also be
             a power series.

           * If x is a vector, complex or quadratic number, then the
             valuation is the minimum of the component valuations.

           * If x = 0, the result is 2^31-1 on 32-bit machines or
             2^63-1 on 64-bit machines if x is an exact object.
             If x is a p-adic number or power series, the result
             is the exponent of the zero.

        INPUT:
            x -- gen
            p -- coercible to gen
        OUTPUT:
            gen -- integer
        EXAMPLES:
            >>> pari(9).valuation(3)
            2
            >>> pari(9).valuation(9)
            1
            >>> x = pari(9).Mod(27); x.valuation(3)
            2
            >>> pari('5/3').valuation(3)
            -1
            >>> pari('9 + 3*x + 15*x^2').valuation(3)
            1
            >>> pari([9,3,15]).valuation(3)
            1
            >>> pari('9 + 3*x + 15*x^2 + O(x^5)').valuation(3)
            1

            >>> pari('x^2*(x+1)^3').valuation(pari('x+1'))
            3
            >>> pari('x + O(x^5)').valuation('x')
            1
            >>> pari('2*x^2 + O(x^5)').valuation('x')
            2

            >> pari(0).valuation(3)   # on 32-bit machine
            2147483647
        """

def foo74():	"""
        variable(x): Return the main variable of the object x, or p
        if x is a p-adic number.

        This function raises a TypeError exception on scalars, i.e.,
        on objects with no variable associated to them.

        INPUT:
            x -- gen 
        OUTPUT:
            gen 
        EXAMPLES:
            >>> pari('x^2 + x -2 ').variable()
            x
            >>> pari('1+2^3 + O(2^5)').variable()
            2
            >>> pari('x+y').variable()
            x
            >>> pari('y+z').variable()
            y
        """

def foo75():	"""
        gamma(x): Gamma function at s.
        INPUT:
            s -- gen (real or complex number
        OUTPUT:
            gen -- value of zeta at s.
        EXAMPLES:
        """

def foo76():	"""
        zeta(s): Riemann zeta function at s with s a complex
                 or a p-adic number.
        INPUT:
            s -- gen (real or complex number
        OUTPUT:
            gen -- value of zeta at s.
        EXAMPLES:
        """

def foo77():	"""
        e.elladd(z1, z2)

        Sum of the points z1 and z2 on the elliptic curve e.

        INPUT:
            e -- elliptic curve E
            z1 -- point on E
            z2 -- point on E
            
        OUTPUT:
            point on E

        EXAMPLES:
        First we create an elliptic curve:
        
            >>> e = pari([0, 1, 1, -2, 0]).ellinit()
            >>> str(e)[:65]   # first part of output
            '[0, 1, 1, -2, 0, 4, -4, 1, -3, 112, -856, 389, 1404928/389, [0.90'
            
        Next we add two points on the elliptic curve.  Notice that
        the Python lists are automatically converted to PARI objects so
        you don't have to do that explicitly in your code.
        
            >>> e.elladd([1,0,1], [-1,1,1])
            [-3/4, -15/8]
        """

def foo78():	"""
        e.ellak(n): Returns the coefficient a_n of the L-function of
        the elliptic curve e, i.e. the coefficient of a newform of
        weight 2 newform.

        The curve e must be a medium or long vector of the type given
        by ellinit. For this function to work for every n and not just
        those prime to the conductor, e must be a minimal Weierstrass
        equation. If this is not the case, use the function
        ellminimalmodel first before using ellak (or you will get
        INCORRECT RESULTS!)

        INPUT:
            e -- a PARI elliptic curve.
            n -- integer ..

        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.ellak(6)
            2
            >>> e.ellak(2005)        
            2
            >>> e.ellak(-1)
            0
            >>> e.ellak(0)
            0
        """

def foo79():	"""
        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.ellan(3)
            [1, -2, -1]
            >>> e.ellan(20)
            [1, -2, -1, 2, 1, 2, -2, 0, -2, -2, 1, -2, 4, 4, -1, -4, -2, 4, 0, 2]
            >>> e.ellan(-1)
            []
        """

def foo80():	"""
        e.ellak(p): Returns the prime-indexed coefficient a_p of the
        L-function of the elliptic curve e, i.e. the coefficient of a
        newform of weight 2 newform.

        If p is not prime, this function will return an INCORRECT
        answer.

        The curve e must be a medium or long vector of the type given
        by ellinit. For this function to work for every n and not just
        those prime to the conductor, e must be a minimal Weierstrass
        equation. If this is not the case, use the function
        ellminimalmodel first before using ellak (or you will get
        INCORRECT RESULTS!)

        INPUT:
            e -- a PARI elliptic curve.
            p -- prime integer ..

        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.ellap(2)
            -2
            >>> e.ellap(2003)
            4
            >>> e.ellak(-1)
            0
        """

def foo81():	"""
        EXAMPLES:
            >>> e = pari([0,1,1,-2,0]).ellinit()
            >>> e.ellbil([1, 0, 1], [-1, 1, 1])
            0.41818898449886058562988945853
            >>> get_real_precision()
            5

        Increasing the precision does not increase the result, since quantities
        related to the elliptic curve were computed to low precision.
            >>> set_real_precision(10)
            >>> e.ellbil([1, 0, 1], [-1, 1, 1])
            0.41818898449886058562988945853
            
        However, if we recompute the elliptic curve after increasing the precision,
        then the bilinear pairing will be computed to higher precision as well.
            >>> e = pari([0,1,1,-2,0]).ellinit()
            >>> e.ellbil([1, 0, 1], [-1, 1, 1])
            0.41818898449886058562988945821587638237833645961491348466736402242784203834353
            >>> set_real_precision(5)
        """

def foo82():	"""
        EXAMPLES:
            >>> e = pari([1,2,3,4,5]).ellinit()
            >>> e.ellglobalred()
            [10351, [1, -1, 0, -1], 1]
            >>> f = e.ellchangecurve([1,-1,0,-1])
            >>> f[:5]
            [1, -1, 0, 4, 3]
        """

def foo83():	"""
        x.ellchangepoint(y): change data on point or vector of points x
                             on an elliptic curve according to y=[u,r,s,t]

        EXAMPLES:
            >>> e = pari([0,1,1,-2,0]).ellinit()
            >>> x = pari([1,0,1])
            >>> e.ellisoncurve([1,4,4])
            False
            >>> e.ellisoncurve(x)
            True
            >>> f = e.ellchangecurve([1,2,3,-1])
            >>> f[:5]   # show only first five entries
            [6, -2, -1, 17, 8]
            >>> x.ellchangepoint([1,2,3,-1])
            [-1, 4]
            >>> f.ellisoncurve([-1,4])
            True
        """

def foo84():	"""
        om.elleisnum(k, {flag=0}, {prec}): om=[om1,om2] being a
            2-component vector giving a basis of a lattice L and k an
            even positive integer, computes the numerical value of the
            Eisenstein series of weight k. When flag is non-zero and
            k=4 or 6, this gives g2 or g3 with the correct
            normalization.

        INPUT:
            om -- gen, 2-component vector giving a basis of a lattice L
            k  -- int (even positive)
            flag -- int (default 0)
            pref -- precision

        OUTPUT:
            gen -- numerical value of E_k

        EXAMPLES:
            >>> e = pari([0,1,1,-2,0]).ellinit()
            >>> om = e.omega()
            >>> om
            [2.4902125608550550753213577920, 1.9717377015516482044224076983*I]
            >>> om.elleisnum(2)
            -5.2886493396542576217915346955
            >>> om.elleisnum(4)
            111.999999999999999999999999968
            >>> om.elleisnum(100)
            2.1531424857607763611270703324 E50
            >>> om.elleisnum(5)
            Traceback (most recent call last):
            ...
            ArithmeticError: k(=5) must be even
            >>> om.elleisnum(0)
            Traceback (most recent call last):
            ...
            ArithmeticError: k(=0) must be positive
        """

def foo85():	"""
        ellheightmatrix(e,x)

        Returns the height matrix for vector of points x on elliptic curve e using
        theta functions.
        """

def foo86():	"""
        ellinit(x,{flag=0}): x being the vector [a1,a2,a3,a4,a6], gives
        the vector: [a1,a2,a3,a4,a6,
                    b2,b4,b6,b8,
                    c4,c6,
                    delta,j,
                    [e1,e2,e3],
                    w1,w2,eta1,eta2,area].
        If the curve is defined over a p-adic field, the last six components
        are replaced by root,u^2,u,q,w,0. If optional flag is 1, omit them
        altogether.
        """

def foo87():	"""
        
        ellminimalmodel(e): return the standard minimal integral model
        of the rational elliptic curve e and the corresponding change
        of variables.
        INPUT:
            e -- gen (that defines an elliptic curve)
        OUTPUT:
            gen -- minimal model
            gen -- change of coordinates
        EXAMPLES:
            >>> e = pari([1,2,3,4,5]).ellinit()
            >>> F, ch = e.ellminimalmodel()
            >>> F[:5]
            [1, -1, 0, 4, 3]
            >>> ch
            [1, -1, 0, -1]
            >>> e.ellchangecurve(ch)[:5]
            [1, -1, 0, 4, 3]
        """

def foo88():	"""
        ellwp(E, z,{flag=0}): Return the complex value of the Weierstrass
        P-function at z on the lattice defined by e.

        INPUT:
            E -- list OR elliptic curve
                  list -- [om1, om2], which are Z-generators for a lattice
                  elliptic curve -- created using ellinit
                   
            z -- (optional) complex number  OR string (default = "z")
                   complex number -- any number in the complex plane
                   string (or PARI variable) -- name of a variable.
                 
            n -- int (optional: default 20) if z is a variable, compute up to at least o(z^n).
            
            flag -- int: 0 (default): compute only P(z)
                         1 compute [P(z),P'(z)]
                         2 consider om or E as an elliptic curve and use P-function to
                           compute the point on E (with the Weierstrass equation for E)
                           P(z)
                           for that curve (identical to ellztopoint in this case).
                    
        OUTPUT:
            gen -- complex number or list of two complex numbers

        EXAMPLES:

        We first define the elliptic curve X_0(11):
            >>> E = pari([0,-1,1,-10,-20]).ellinit()

        Compute P(1).
            >>> E.ellwp(1)
            13.965869525748497794694977693 + 2.660659330 E-28*I

        Compute P(1+I), where I = sqrt(-1).
            >>> E.ellwp(pari("1+I"))
            -1.1151068256555508792090664874 + 2.3341905230746988361847987997*I
            >>> E.ellwp("1+I")
            -1.1151068256555508792090664874 + 2.3341905230746988361847987997*I

        The series expansion, to the default 20 precision:
            >>> E.ellwp()
            z^-2 + 31/15*z^2 + 2501/756*z^4 + 961/675*z^6 + 77531/41580*z^8 + 1202285717/928746000*z^10 + 2403461/2806650*z^12 + 30211462703/43418875500*z^14 + 3539374016033/7723451736000*z^16 + 413306031683977/1289540602350000*z^18 + O(z^20)

        Compute the series for wp to lower precision:
            >>> E.ellwp(n=4)
            z^-2 + 31/15*z^2 + O(z^4)

        Next we use the version where the input is generators for a lattice:
            >>> pari([1.2692, '0.63 + 1.45*I']).ellwp(1)
            13.965614693668943648020037361 + 0.00064482927281053614745416359791*I

        With flag 1 compute the pair P(z) and P'(z):
            >>> E.ellwp(1, flag=1)
            [13.965869525748497794694977693 + 2.660659330 E-28*I, 50.561930088007275255584656888 + 1.615587134 E-27*I]

        With flag=2, the computed pair is (x,y) on the curve instead of [P(z),P'(z)]:
            >>> E.ellwp(1, flag=2)
            [14.299202859081831128028311026 + 2.660659330 E-28*I, 50.061930088007275255584656888 + 1.615587134 E-27*I]
        """

def foo89():	"""
        e.omega(): return basis for the period lattice of the elliptic curve e.
        
        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.omega()
            [1.2692093042795534216887946168, 0.63460465213977671084439730838 + 1.4588166169384952293308896122*I]
        """

def foo90():	"""
        e.disc(): return the discriminant of the elliptic curve e.
        
        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.disc()
            -161051
            >>> _.factor()
            [-1, 1; 11, 5]
        """

def foo91():	"""
        e.j(): return the j-invariant of the elliptic curve e.
        
        EXAMPLES:
            >>> e = pari([0, -1, 1, -10, -20]).ellinit()
            >>> e.j()
            -122023936/161051
            >>> _.factor()
            [-1, 1; 2, 12; 11, -5; 31, 3]
        """

def foo92():	"""
        polgalois(x): Galois group of the polynomial x
        """

def foo93():	"""
        polisirreducible(x): Returns True if x is an irreducible
        non-constant polynomial, or False if x is reducible or
        constant.
        """

def foo94():	"""
        polroots(x,{flag=0}): complex roots of the polynomial x. flag
        is optional, and can be 0: default, uses Schonhage's method modified
        by Gourdon, or 1: uses a modified Newton method.
        """

def foo95():	"""
        factor(x,{lim}): factorization of x. lim is optional and can
        be set whenever x is of (possibly recursive) rational
        type. If lim is set return partial factorization, using
        primes up to lim (up to primelimit if lim=0)
        """

def foo96():	"""
        eint1(x,{n})

        Returns the exponential integral E1(x). If the optional
        argument n is given, computes the vector of the first n
        values of the exponential integral E1(x*n), for x > 0.
        
        REFERENCE: See page 262, Prop 5.6.12, of Cohen's book
        "A Course in Computational Algebraic Number Theory".
        """

def foo97():	"""
        """

def foo98():	"""
        """

def foo99():	"""
        """

def foo100():	"""
        charpoly(A,{v=x},{flag=0}): det(v*Id-A) = characteristic polynomial of the
        matrix A using the comatrix. flag is optional and may be set to 1 (use
        Lagrange interpolation) or 2 (use Hessenberg form), 0 being the default.
        """

def foo101():	"""
        nextprime(x): smallest pseudoprime >= x
        """

def foo102():	"""
        EXAMPLES:
           >>> x = pari("x"); y = pari("y")
           >>> f = x**3 + 17*x + 3
           >>> f.subst(x, y)
           y^3 + 17*y + 3
           >>> f.subst(x, "z")
           z^3 + 17*z + 3
           >>> f.subst(x, "z")**2
           z^6 + 34*z^4 + 6*z^3 + 289*z^2 + 102*z + 9
           >>> f.subst(x, "x+1")
           x^3 + 3*x^2 + 20*x + 21
           >>> f.subst(x, "xyz")
           xyz^3 + 17*xyz + 3
           >>> f.subst(x, "xyz")**2
           xyz^6 + 34*xyz^4 + 6*xyz^3 + 289*xyz^2 + 102*xyz + 9
        """

def foo103():	"""
    Create a new gen, but don't free any memory on the stack.
    """

def foo104():	"""
##     Create a new gen, then free everything on the stack below av.
##     The gen *must* not reference anything on the stack above av.
##     """

def foo105():	"""
    Create a new gen, then free the *entire* stack.
    """

def foo106():	"""
    Create the PARI object got by evaluating s using PARI.
    """

def foo107():	"""
    Converts a Python string into a PARI variable reference number.
    Or if v = -1, returns -1.
    """

def foo108():	"""
    Stack size, in bytes
    """

def foo109():	"""
    How much memory in megabytes is currently being used on the PARI stack.
    """

def foo110():	"""
    How much memory in bytes is currently available on the PARI stack.
    """

def foo111():	"""
    prime_list(n): returns list of the first n primes

    INPUT:
        n -- C long
    OUTPUT:
        gen -- PARI list of first n primes
    EXAMPLES:
        >>> prime_list(0)
        []
        >>> prime_list(-1)
        []
        >>> prime_list(3)
        [2, 3, 5]
        >>> prime_list(10)
        [2, 3, 5, 7, 11, 13, 17, 19, 23, 29]
        >>> prime_list(20)
        [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71]
        >>> len(prime_list(1000))
        1000
    """

def foo112():	"""
    nth_prime(n): returns the n-th prime, where n is a C-int
    """

def foo113():	"""
    EXAMPLES:
        >> euler()
        0.57721566490153286060651209008
        >> get_real_precision()
        5
        >> euler(6)
        0.57721566490153286060651209008240243104
        >> set_real_precision(10)
        >> euler()
        0.57721566490153286060651209008240243104215933593992359880576723488486772677767
        
    We restore precision to the default.
        >>> set_real_precision(5)
        
    Note that euler is still saved to the largest precision to which it has been
    computed so far:
        >> euler()
        0.57721566490153286060651209008240243104215933593992359880576723488486772677767
    """

def foo114():	"""
    EXAMPLES:
        >> set_real_precision(5)
        >> pi()
        3.1415926535897932384626433833
        >> pi(6)
        3.14159265358979323846264338327950288419
        >> get_real_precision()
        5
        >> set_real_precision(10)
        >> pi()
        3.1415926535897932384626433832795028841971693993751058209749445923078164062862
        
    We restore precision to the default.
        >> set_real_precision(5)
        
    Note that pi is still saved to the largest precision to which it has been
    computed so far:
        >> pi()
        3.1415926535897932384626433832795028841971693993751058209749445923078164062862
    """

def foo115():	"""
    
    finitefield_init(p, long n, var="x"): Return a polynomial f(x) so
    that the extension of F_p of degree n is k = F_p[x]/(f).  Moreover,
    the element x (mod f) of k is a generator for the multiplicative
    group of k.
    
    INPUT:
        p -- int, a prime number
        n -- int, positive integer
        var -- str, default to "x", but could be any pari variable. 
    OUTPUT:
        pari polynomial mod p -- defines field
    EXAMPLES:
        >>> finitefield_init(97,1)
        Mod(1, 97)*x + Mod(92, 97)

    The last entry in each of the following two lines is
    determined by a random algorithm.
        >> finitefield_init(7,2)    
        Mod(1, 7)*x^2 + Mod(6, 7)*x + Mod(3, 7)
        >> finitefield_init(2,3)
        Mod(1, 2)*x^3 + Mod(1, 2)*x^2 + Mod(1, 2)
    """

def foo116():	"""
    Set the default precision to n words (word = 4 bytes on
    a 32-bit machine). 
    
    EXAMPLES:
        >>> get_real_precision()
        5
        >>> set_real_precision(10)
        >>> get_real_precision()
        10
        >>> set_real_precision(5)        
    """

def foo117():	"""
    EXAMPLES:
        >>> get_real_precision()
        5
        >>> set_real_precision(10)
        >>> get_real_precision()
        10
        >>> set_real_precision(5)
    """

def foo118():	"""
    Set the default precision used when coercing objects into power series.
    
    EXAMPLES:
        >>> default = get_series_precision()
        >>> f = pari('1+x + x^2 + x^3')
        >>> set_series_precision(3)
        >>> f.exp()
        2.7182818284590452353602874714 + 2.7182818284590452353602874714*x + 4.0774227426885678530404312070*x^2 + 5.8896106283279313432806228546*x^3 + O(x^4)
        >>> set_series_precision(10)
        >>> f.exp()
        2.7182818284590452353602874714 + 2.7182818284590452353602874714*x + 4.0774227426885678530404312070*x^2 + 5.8896106283279313432806228546*x^3 + 5.5498253997705506888605869206*x^4 + 5.9122629768984233869086252502*x^5 + 5.7801242769038864657452779426*x^6 + 4.8934466328589121865920413149*x^7 + 4.2738105146702444098510591188*x^8 + 3.4890085123444142422523306196*x^9 + O(x^10)
        >>> set_series_precision(default)
    """

def foo119():	"""
    Get the default precision used when coercing objects into power series.
    
    EXAMPLES:
        >>> get_series_precision()
        16
    """

def foo120():	"""
    Recompute the primes table including at least all primes up to M.
    
    EXAMPLES:
        >>> init_primes(200000)
    """

def foo121():	"""
    Read a script from the named filename into the interpreter, where
    s is a string.  The functions defined in the script are then
    available for use from SAGE/PARI.

    EXAMPLE:
    
        If foo.gp is a script that contains
            {foo(n) =
                n^2
            }
        and you type read("foo.gp"), then the command
        pari("foo(12)") will create the Python/PARI gen which
        is the integer 144.

    CONSTRAINTS:    
        The PARI script must *not* contain the following function calls:

             print, default, ???    (please report any others that cause trouble)

        Also multiline functions should be written in the following form:

            {foo(x) =
                code...
            }

        and *NOT* in the form

            foo(x) =
            {
                    code...
            }
    """


if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
