"""
Polylist functionality.
"""

#*****************************************************************************
#       Copyright (C) 2005 David Joyner (2005-06-21, wdj@usna.edu)
#                  and 2005 William Stein (was@math.harvard.edu)
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

"""
SUMMARY:
    The functions in this file use the "distributive representation"
    of multivariate polynomials

        [[c1,[e1,...,er]],...] <-> c1*x1^e1*...*xr^er+...,

    which we call a polylist.   This file implements arithmetic
    functionality for polylists.

EXAMPLES:
    >>> import sage.rings.polylist_fcns as PList
    >>> polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
    >>> polylist2=[[-1,[1,3]],[-3,[1,2]],[2,[1,1]]]
    >>> P1=PList.PolyList(polylist1)
    >>> P2=PList.PolyList(polylist2)
    >>> P1+P2
    PolyList:[[2, [2, 3]], [4, [2, 1]], [-1, [1, 3]], [0, [1, 2]], [2, [1, 1]]]
    >>> P3=P1+P2
    >>> print P3
    2x_0^2x_1^3 + 4x_0^2x_1 + -1x_0x_1^3x_0x_1^2 + 2x_0x_1
    >>> P4=P1*P2
    >>> P4
    PolyList with distributtive representation:[[-2, [3, 6]], [-6, [3, 5]], [0, [3, 4]], [-3, [2, 5]], [-9, [2, 4]], [6, [2, 3]], [-12, [3, 3]], [8, [3, 2]]]
    >>> print P4
    -2x_0^3x_1^6 + -6x_0^3x_1^5x_0^3x_1^4 + -3x_0^2x_1^5 + -9x_0^2x_1^4 + 6x_0^2x_1^3 + -12x_0^3x_1^3 + 8x_0^3x_1^2
    >>> polylist5=[[2,[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]],[3,[1,0,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]],[4,[2,1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20]]]
    >>> P5=PList.PolyList(polylist5)
    >>> P5
    PolyList with distributive representation:[[2, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]], [3, [1, 0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]], [4, [2, 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20]]]
    >>> print P5
    2x_0x_1^2x_2^3x_3^4x_4^5x_5^6x_6^7x_7^8x_8^9x_9^10x_10^11x_11^12x_12^13x_13^14x_14^15x_15^16x_16^17x_17^18x_18^19x_19^20 + 3x_0x_1^0x_2^3x_3^4x_4^5x_5^6x_6^7x_7^8x_8^9x_9^10x_10^11x_11^12x_12^13x_13^14x_14^15x_15^16x_16^17x_17^18x_18^19x_19^20 + 4x_0^2x_1x_2^3x_3^4x_4^5x_5^6x_6^7x_7^8x_8^9x_9^10x_10^11x_11^12x_12^13x_13^14x_14^15x_15^16x_16^17x_17^18x_18^19x_19^20

AUTHOR:  David Joyner
SECONDARY AUTHOR: -- William Stein (minor modifications)
"""

from sage.rings.rational_field import RationalField

def flatten(lists):
    """
    used to convert [[row1],[row2],...,[rowk]] matrix notation
    to [row1,...,rowk] (in fact, row sizes need not be the same)
 
    EXAMPLES:
        >>> flatten([[1,2],[3,4]])
        [1,2,3,4]
    """
    return sum(lists,[])

def list_union(self, other):
    """
    INPUT:
        self -- list
        other -- list
    OUTPUT:
        list -- their union, with order preserved
    """
    res = self[:]
    for x in other:
       if not(x in res):
           res.append(x)
    return res

def list_intersection(self, other):
    """
    INPUT: self, other - any sequence
    OUTPUT: their intersection (again, order preserved)
    """ 
    res = self[:]
    cap = []
    for x in other:
        if x in res:
            cap.append(x)
    return cap


def list_filtered(L,boolprop):
    """
    INPUT:
        L -- list
        boolprop -- a Boolean-valued function on the elements of list
    """
    return [x for x in L if boolprop(x)]

def list_positions(L,x):
    """
    INPUT: L is a list
          x is anything
    OUTPUT: the list of indices i where x = L[i]
    """
    ans=[]
    n=len(L)
    for i in range(n):
        if x==L[i]:
            ans.append(i)
    return ans
    # WS: If you do this more than a few times on a single list it
    # WS: would be better that this list is sorted, and then one
    # WS: would use sage.misc.search

def list_total(L):
    """
    INPUT: L is a list
    OUTPUT: returns the sum of its entries
    """
    return sum(L,0)   # WS: The extra 0 is so the empty list returns a sum of 0.

def vector_add(list1,list2):
    """
    INPUT: list1,list2 are lists of numbers of same length
    OUTPUT: the component-wise sum, as though they were vectors
    """
    n=len(list1)           #does not check if lengths agree
    return [(list1[i]+list2[i]) for i in range(n)]


def lex_monord(left,right):
    """
    left,right are monomial polylists
    returns comp fcn (i.e., takes values 1, 0, -1) on polylists

    EXAMPLES:
        polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
        polylist1.sort(lex_monord)
        polylist1
        #  [[2, [2, 3]], [4, [2, 1]], [3, [1, 2]]]
    """
    p=left[1]
    q=right[1]
    if p == q:
        return 0
    ans = p<q
    return int(2*int(ans)-1)


# WS: this was a member function of PolyList, which made it not work
# in degrevlex_monord below.
def standard_basis_vector(a,i,n):
    """
    returns basis n-vector with a in ith position
    (a is an integer).
    """
    return [a*int(j==i-1) for j in range(n)]

def degrevlex_monord(left,right):
    # Do the linear algebra import during definition of function to avoid circular import
    import sage.linalg.vector_space, sage.linalg.matrix_space
    p = left[1]
    q = right[1]
    r = len(p)  # assume this  =  len(q)
    V = sage.linalg.vector_space.VectorSpace(RationalField(),r)
    p = V.vector(p)
    q = V.vector(q)
    M = sage.linalg.matrix_space.MatrixSpace(RationalField(),r,r)
    B = [[1 for i in range(r)]]
    for i in range(r-1):
        B.append(standard_basis_vector(-1,r-i,r))

    A = M.matrix(flatten(B))  # This is the matrix repn of the degrevlex
    At = A.transpose()
    p = p*At                  # monomial ordering - see p52 of 
    q = q*At                  # Computational Commutative Algebra 1,
                              # M. Kreuzer, L. Robbiano.
    if p == q:
        return 0
    ans = (p < q)
    return int(2*int(ans)-1)

def coeffs_polylist(polylist):
    n = len(polylist)
    coeffs = [polylist[i][0] for i in range(n)]
    return coeffs

def expons_polylist(polylist):
    n = len(polylist)
    expons = [polylist[i][1] for i in range(n)]
    return expons

def AddPolylists(left, right,monord=lex_monord):
    """
         Uses the "distributive representation" of polynomials
         left=[[c1,[e1,...,er]],...] <-> c1*x1^e1*...*xr^er+...

         EXAMPLES:
         (collection is illustrated)

         polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
         polylist2=[[-1,[1,3]],[-3,[1,2]],[2,[1,1]]]
         AddPolylists(polylist1,polylist2,lex_monord)
         #     [[2, [2, 3]], [4, [2, 1]], [-1, [1, 3]], [0, [1, 2]], [2, [1, 1]]]
    """
    n1=len(left)
    n2=len(right)
    if n1==1 and left[0][0]==0:
        return right
    if n2==1 and right[0][0]==0:
        return left
    coeffs1=coeffs_polylist(left)
    coeffs2=coeffs_polylist(right)
    expons1=expons_polylist(left)
    expons2=expons_polylist(right)
    newexpons=list_union(expons1,expons2)
    # these are all the pure monomials which occur in either polys
    dblexpons=list_intersection(expons1,expons2)
    # these are the monomials which occur in both polys
    n=len(newexpons)
    newpolylist=[]
    for i in range(n):
        if newexpons[i] in dblexpons:
            i1=expons1.index(newexpons[i])
            i2=expons2.index(newexpons[i])
            coeff=coeffs1[i1]+coeffs2[i2]                # new coeff (when pure monomials match)
            newpolylist.append([coeff,newexpons[i]]) # leave in if coeff=0???
            if not(newexpons[i] in dblexpons):
                if (newexpons[i] in expons1):
                    i1=expons1.index(newexpons[i])
                    newpolylist.append([coeffs1[i1],newexpons[i]])
                if (newexpons[i] in expons2):
                    i2=expons2.index(newexpons[i])
                    newpolylist.append([coeffs2[i2],newexpons[i]])
    newpolylist.sort(monord)
    return newpolylist

def MultPolylists(left, right,monord=lex_monord):
    """
    Uses the "distributive representation" of polynomials
    left=[[c1,[e1,...,er]],...] <-> c1*x1^e1*...*xr^er+...

    EXAMPLE:
    (illustrates cancellation)

    polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
    polylist2=[[-1,[1,3]],[-3,[1,2]],[2,[1,1]]]
    mult_polylists(polylist1,polylist2)
    # [[-2, [3, 6]], [-6, [3, 5]], [[0, [3, 4]]], [-3, [2, 5]],
    #  [-9, [2, 4]], [6, [2, 3]], [-12, [3, 3]], [8, [3, 2]]]
    """
    zero=0                        # 0 in **coeff ring** of polys
    n1=len(left)
    n2=len(right)
    r=len(left[0][1])                 # no checking if all expons of left,right have same size
    coeffs1=coeffs_polylist(left)
    coeffs2=coeffs_polylist(right)
    expons1=expons_polylist(left)
    expons2=expons_polylist(right)
    newpolylist=[]
    for lpoly in left:                           # mult together left*right - no collecting
        for rpoly in right:
            newpolylist.append([lpoly[0]*rpoly[0],vector_add(lpoly[1],rpoly[1])])
    newcoeffs=coeffs_polylist(newpolylist)
    newexpons=expons_polylist(newpolylist)
    multpoly=[]
    expons=[]
    for poly in newpolylist:
        num_occur=newexpons.count(poly[1])         # number times a given "pure" monomial occurs
        if num_occur==1:
            multpoly.append(poly)
            expons.append(poly[1])
        if num_occur>1 and not(poly[1] in expons):    # collecting new "like" terms
            pos=list_positions(newexpons,poly[1])     # positions where pure monomial occurs
            tpoly=[[zero,poly[1]]]                     # 0 (**in coeff ring**) times pure monomial
            for i in pos:
                tpoly=AddPolylists(tpoly,[newpolylist[i]])
            expons.append(poly[1])
            multpoly.append(tpoly[0])
    #  multpoly.sort(monord)
    return multpoly


class PolyList:
    def __init__(self,plist,morder=lex_monord):
        self.distrepn = plist           # the distributive representation
        n=len(self.distrepn)
        self.coefficients=[plist[i][0] for i in range(n)]
        self.exps=[plist[i][1] for i in range(n)]
        self.monomialordering=morder
        
    def intersect(self, other):        # other is any sequence
        res = []                       # self is the subject
        for x in self.distrepn:
            if x in other:             # pick common items
                res.append(x)
        return PolyList(res)                # return a new Set

    def union(self, other):            # other is any sequence
        res = self.distrepn[:]             # copy of my list
        for x in other:                # add items in other
            if not x in res:
                res.append(x)
        return PolyList(res)

    def concat(self, value):           # value: list, Set...
        for x in value:                # removes duplicates
            if not x in self.distrepn:
                self.distrepn.append(x)

    def __len__(self):
        return len(self.distrepn)        # len(self)

    def __getitem__(self, key):
        return self.distrepn[key]        # self[i]

    def __or__(self, other):
        return self.union(other)     # self | other

    def __repr__(self):
        return 'PolyList with distributive representation:' + `self.distrepn`  # print

    def __str__(self):
        nterms=len(self)
        nvars=len(self.exps[0])
        poly=""
        for i in range(nterms):
            if self.distrepn[i][0]<>0:
                if i>0:
                    poly=poly + " + %g"%self.distrepn[i][0]
                else:
                    poly=poly + "%g"%self.distrepn[i][0]
                for j in range(nvars):
                    if self.distrepn[i][1][j]==0:
                        pass
                    elif self.distrepn[i][1][j]==1:
                        poly=poly+"x_%d" %j
                    else:
                        poly=poly+"x_%d^%s" %(j,self.distrepn[i][1][j])
            else:
                pass
        return poly

    # WS: Member functions should all have first argument self (or at
    # least something of type PolyList).  If they don't, then they
    # shouldn't be member functions.
    def dirac_delta(i,j):
        """
        input: i,j, objects
        output: 1 if i==j, 0 otherwise
        """
        return int(i==j)

    def add_polylists(left, right, monord = lex_monord):
        """
        Uses the "distributive representation" of polynomials
        left=[[c1,[e1,...,er]],...] <-> c1*x1^e1*...*xr^er+...

        EXAMPLES:
        (collection is illustrated)

        polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
        polylist2=[[-1,[1,3]],[-3,[1,2]],[2,[1,1]]]
        add_polylists(polylist1,polylist2,lex_monord)
        #     [[2, [2, 3]], [4, [2, 1]], [-1, [1, 3]], [0, [1, 2]], [2, [1, 1]]]
        """
        n1=len(left)
        n2=len(right)
        if n1==1 and left.distrepn[0][0]==0:
         return right
        if n2==1 and right.distrepn[0][0]==0:
         return left
        coeffs1=left.coefficients
        coeffs2=right.coefficients
        expons1=left.exps
        expons2=right.exps
        newexpons=list_union(expons1,expons2)
        # these are all the pure monomials which occur in either polys
        dblexpons=list_intersection(expons1,expons2)
        # these are the monomials which occur in both polys
        n=len(newexpons)
        newpolylist=[]
        for i in range(n):
            if newexpons[i] in dblexpons:
                i1=expons1.index(newexpons[i])
                i2=expons2.index(newexpons[i])
                coeff=coeffs1[i1]+coeffs2[i2]                # new coeff (when pure monomials match)
                newpolylist.append([coeff,newexpons[i]])     # leave in if coeff=0???
            if not(newexpons[i] in dblexpons):
                if (newexpons[i] in expons1):
                    i1=expons1.index(newexpons[i])
                    newpolylist.append([coeffs1[i1],newexpons[i]])
                if (newexpons[i] in expons2):
                    i2=expons2.index(newexpons[i])
                    newpolylist.append([coeffs2[i2],newexpons[i]])
        newpolylist.sort(monord)
        return PolyList(newpolylist)
     
    def __add__(self, other):
        return self.add_polylists(other) # self + other

    def mult_polylists(left, right,monord =lex_monord):
        """
        Uses the "distributive representation" of polynomials
        left=[[c1,[e1,...,er]],...] <-> c1*x1^e1*...*xr^er+...

        EXAMPLE:
        (illustrates cancellation)

        polylist1=[[2,[2,3]],[3,[1,2]],[4,[2,1]]]
        polylist2=[[-1,[1,3]],[-3,[1,2]],[2,[1,1]]]
        mult_polylists(polylist1,polylist2)
        # [[-2, [3, 6]], [-6, [3, 5]], [[0, [3, 4]]], [-3, [2, 5]],
        #  [-9, [2, 4]], [6, [2, 3]], [-12, [3, 3]], [8, [3, 2]]]
        """
        zero=0                        # 0 in **coeff ring** of polys
        n1=len(left)
        n2=len(right)
        r=len(left.distrepn[0][1])        # no checking if all expons of left,right have same size
        coeffs1=left.coefficients
        coeffs2=right.coefficients
        expons1=left.exps
        expons2=right.exps
        newpolylist=[]
        for lpoly in left.distrepn:                     # mult together left*right - no collecting
            for rpoly in right.distrepn:
                newpolylist.append([lpoly[0]*rpoly[0],vector_add(lpoly[1],rpoly[1])])
        newcoeffs=PolyList(newpolylist).coefficients
        newexpons=PolyList(newpolylist).exps
        multpoly=[]
        expons=[]
        for poly in newpolylist:
            num_occur=newexpons.count(poly[1])         # number times a given "pure" monomial occurs
            if num_occur==1:
                multpoly.append(poly)
                expons.append(poly[1])
            if num_occur>1 and not(poly[1] in expons):    # collecting new "like" terms
                pos=list_positions(newexpons,poly[1])     # positions where pure monomial occurs
                tpoly=[[zero,poly[1]]]                     # 0 (**in coeff ring**) times pure monomial
                for i in pos:
                    tpoly=PolyList.add_polylists(PolyList(tpoly),PolyList([newpolylist[i]]))
                expons.append(poly[1])
                multpoly.append(tpoly)
        #       list(multpoly).sort(monord)
        return PolyList(multpoly)

    def __mul__(self, other):
        return PolyList(MultPolylists(self.distrepn,other.distrepn)) # self * other


# doctests
if __name__ ==  '__main__':
    import doctest, sys
    # Run the doctests.
    doctest.testmod(sys.modules[__name__])
